@@ -17,36 +17,24 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-/**
-   @file sampler.c Sampler Plugin
-
-   A simple example of an LV2 sampler that dynamically loads a single sample
-   (based on incoming events) and triggers their playback (based on incoming
-   MIDI note events).
-
-   This plugin illustrates:
-   - UI <=> Plugin communication via events
-   - Use of the worker extension for non-realtime tasks (sample loading)
-   - Use of the log extension to print log messages via the host
-   - Saving plugin state via the state extension
-   - Dynamic plugin control via the same properties saved to state
-*/
-
 #include <math.h>
-#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef __cplusplus
+#    include <stdbool.h>
+#endif
 
 #include <sndfile.h>
 
 #include "lv2/lv2plug.in/ns/ext/atom/forge.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
 #include "lv2/lv2plug.in/ns/ext/log/log.h"
+#include "lv2/lv2plug.in/ns/ext/log/logger.h"
+#include "lv2/lv2plug.in/ns/ext/midi/midi.h"
 #include "lv2/lv2plug.in/ns/ext/patch/patch.h"
 #include "lv2/lv2plug.in/ns/ext/state/state.h"
 #include "lv2/lv2plug.in/ns/ext/urid/urid.h"
 #include "lv2/lv2plug.in/ns/ext/worker/worker.h"
-#include "lv2/lv2plug.in/ns/ext/midi/midi.h"
 #include "lv2/lv2plug.in/ns/lv2core/lv2.h"
 
 #include "./uris.h"
@@ -60,60 +48,48 @@ enum {
 static const char* default_sample_file = "click.wav";
 
 typedef struct {
-	SF_INFO info;      /**< Info about sample from sndfile */
-	float*  data;      /**< Sample data in float */
-	char*   path;      /**< Path of file */
-	size_t  path_len;  /**< Length of path */
+	SF_INFO  info;      // Info about sample from sndfile
+	float*   data;      // Sample data in float
+	char*    path;      // Path of file
+	uint32_t path_len;  // Length of path
 } Sample;
 
 typedef struct {
-	/* Features */
+	// Features
 	LV2_URID_Map*        map;
 	LV2_Worker_Schedule* schedule;
 	LV2_Log_Log*         log;
 
-	/* Forge for creating atoms */
+	// Forge for creating atoms
 	LV2_Atom_Forge forge;
 
-	/* Sample */
-	Sample* sample;
+	// Logger convenience API
+	LV2_Log_Logger logger;
 
-	/* Ports */
+	// Sample
+	Sample* sample;
+	bool    sample_changed;
+
+	// Ports
 	const LV2_Atom_Sequence* control_port;
 	LV2_Atom_Sequence*       notify_port;
 	float*                   output_port;
 
-	/* Forge frame for notify port (for writing worker replies). */
+	// Forge frame for notify port (for writing worker replies)
 	LV2_Atom_Forge_Frame notify_frame;
 
-	/* URIs */
+	// URIs
 	SamplerURIs uris;
 
-	/* Current position in run() */
+	// Current position in run()
 	uint32_t frame_offset;
 
-	/* Playback state */
+	// Playback state
+	float      gain;
 	sf_count_t frame;
 	bool       play;
 } Sampler;
 
-/**
-   Print an error message to the host log if available, or stderr otherwise.
-*/
-LV2_LOG_FUNC(3, 4)
-static void
-print(Sampler* self, LV2_URID type, const char* fmt, ...)
-{
-	va_list args;
-	va_start(args, fmt);
-	if (self->log) {
-		self->log->vprintf(self->log->handle, type, fmt, args);
-	} else {
-		vfprintf(stderr, fmt, args);
-	}
-	va_end(args);
-}
-
 /**
    An atom-like message used internally to apply/free samples.
 
@@ -137,37 +113,34 @@ typedef struct {
 static Sample*
 load_sample(Sampler* self, const char* path)
 {
-	const size_t path_len  = strlen(path);
+	const size_t path_len = strlen(path);
 
-	print(self, self->uris.log_Trace,
-	      "Loading sample %s\n", path);
+	lv2_log_trace(&self->logger, "Loading sample %s\n", path);
 
 	Sample* const  sample  = (Sample*)malloc(sizeof(Sample));
 	SF_INFO* const info    = &sample->info;
 	SNDFILE* const sndfile = sf_open(path, SFM_READ, info);
 
 	if (!sndfile || !info->frames || (info->channels != 1)) {
-		print(self, self->uris.log_Error,
-		      "Failed to open sample '%s'.\n", path);
+		lv2_log_error(&self->logger, "Failed to open sample '%s'\n", path);
 		free(sample);
 		return NULL;
 	}
 
-	/* Read data */
+	// Read data
 	float* const data = malloc(sizeof(float) * info->frames);
 	if (!data) {
-		print(self, self->uris.log_Error,
-		      "Failed to allocate memory for sample.\n");
+		lv2_log_error(&self->logger, "Failed to allocate memory for sample\n");
 		return NULL;
 	}
 	sf_seek(sndfile, 0ul, SEEK_SET);
 	sf_read_float(sndfile, data, info->frames);
 	sf_close(sndfile);
 
-	/* Fill sample struct and return it. */
+	// Fill sample struct and return it
 	sample->data     = data;
 	sample->path     = (char*)malloc(path_len + 1);
-	sample->path_len = path_len;
+	sample->path_len = (uint32_t)path_len;
 	memcpy(sample->path, path, path_len + 1);
 
 	return sample;
@@ -177,7 +150,7 @@ static void
 free_sample(Sampler* self, Sample* sample)
 {
 	if (sample) {
-		print(self, self->uris.log_Trace, "Freeing %s\n", sample->path);
+		lv2_log_trace(&self->logger, "Freeing %s\n", sample->path);
 		free(sample->path);
 		free(sample->data);
 		free(sample);
@@ -201,23 +174,23 @@ work(LV2_Handle                  instance,
 	Sampler*        self = (Sampler*)instance;
 	const LV2_Atom* atom = (const LV2_Atom*)data;
 	if (atom->type == self->uris.eg_freeSample) {
-		/* Free old sample */
+		// Free old sample
 		const SampleMessage* msg = (const SampleMessage*)data;
 		free_sample(self, msg->sample);
 	} else {
-		/* Handle set message (load sample). */
+		// Handle set message (load sample).
 		const LV2_Atom_Object* obj = (const LV2_Atom_Object*)data;
 
-		/* Get file path from message */
+		// Get file path from message
 		const LV2_Atom* file_path = read_set_file(&self->uris, obj);
 		if (!file_path) {
 			return LV2_WORKER_ERR_UNKNOWN;
 		}
 
-		/* Load sample. */
+		// Load sample.
 		Sample* sample = load_sample(self, LV2_ATOM_BODY_CONST(file_path));
 		if (sample) {
-			/* Loaded sample, send it to run() to be applied. */
+			// Loaded sample, send it to run() to be applied.
 			respond(handle, sizeof(sample), &sample);
 		}
 	}
@@ -242,13 +215,13 @@ work_response(LV2_Handle  instance,
 	SampleMessage msg = { { sizeof(Sample*), self->uris.eg_freeSample },
 	                      self->sample };
 
-	/* Send a message to the worker to free the current sample */
+	// Send a message to the worker to free the current sample
 	self->schedule->schedule_work(self->schedule->handle, sizeof(msg), &msg);
 
-	/* Install the new sample */
+	// Install the new sample
 	self->sample = *(Sample*const*)data;
 
-	/* Send a notification that we're using a new sample. */
+	// Send a notification that we're using a new sample.
 	lv2_atom_forge_frame_time(&self->forge, self->frame_offset);
 	write_set_file(&self->forge, &self->uris,
 	               self->sample->path,
@@ -284,14 +257,14 @@ instantiate(const LV2_Descriptor*     descriptor,
             const char*               path,
             const LV2_Feature* const* features)
 {
-	/* Allocate and initialise instance structure. */
+	// Allocate and initialise instance structure.
 	Sampler* self = (Sampler*)malloc(sizeof(Sampler));
 	if (!self) {
 		return NULL;
 	}
 	memset(self, 0, sizeof(Sampler));
 
-	/* Get host features */
+	// Get host features
 	for (int i = 0; features[i]; ++i) {
 		if (!strcmp(features[i]->URI, LV2_URID__map)) {
 			self->map = (LV2_URID_Map*)features[i]->data;
@@ -302,18 +275,19 @@ instantiate(const LV2_Descriptor*     descriptor,
 		}
 	}
 	if (!self->map) {
-		print(self, self->uris.log_Error, "Missing feature urid:map.\n");
+		lv2_log_error(&self->logger, "Missing feature urid:map\n");
 		goto fail;
 	} else if (!self->schedule) {
-		print(self, self->uris.log_Error, "Missing feature work:schedule.\n");
+		lv2_log_error(&self->logger, "Missing feature work:schedule\n");
 		goto fail;
 	}
 
-	/* Map URIs and initialise forge */
+	// Map URIs and initialise forge/logger
 	map_sampler_uris(self->map, &self->uris);
 	lv2_atom_forge_init(&self->forge, self->map);
+	lv2_log_logger_init(&self->logger, self->map, self->log);
 
-	/* Load the default sample file */
+	// Load the default sample file
 	const size_t path_len    = strlen(path);
 	const size_t file_len    = strlen(default_sample_file);
 	const size_t len         = path_len + file_len;
@@ -337,6 +311,9 @@ cleanup(LV2_Handle instance)
 	free(self);
 }
 
+/** Define a macro for converting a gain in dB to a coefficient. */
+#define DB_CO(g) ((g) > -90.0f ? powf(10.0f, (g) * 0.05f) : 0.0f)
+
 static void
 run(LV2_Handle instance,
     uint32_t   sample_count)
@@ -347,16 +324,25 @@ run(LV2_Handle instance,
 	sf_count_t   pos         = 0;
 	float*       output      = self->output_port;
 
-	/* Set up forge to write directly to notify output port. */
+	// Set up forge to write directly to notify output port.
 	const uint32_t notify_capacity = self->notify_port->atom.size;
 	lv2_atom_forge_set_buffer(&self->forge,
 	                          (uint8_t*)self->notify_port,
 	                          notify_capacity);
 
-	/* Start a sequence in the notify output port. */
+	// Start a sequence in the notify output port.
 	lv2_atom_forge_sequence_head(&self->forge, &self->notify_frame, 0);
 
-	/* Read incoming events */
+	// Send update to UI if sample has changed due to state restore
+	if (self->sample_changed) {
+		lv2_atom_forge_frame_time(&self->forge, 0);
+		write_set_file(&self->forge, &self->uris,
+		               self->sample->path,
+		               self->sample->path_len);
+		self->sample_changed = false;
+	}
+
+	// Read incoming events
 	LV2_ATOM_SEQUENCE_FOREACH(self->control_port, ev) {
 		self->frame_offset = ev->time.frames;
 		if (ev->body.type == uris->midi_Event) {
@@ -370,25 +356,57 @@ run(LV2_Handle instance,
 			default:
 				break;
 			}
-		} else if (is_object_type(uris, ev->body.type)) {
-			const LV2_Atom_Object* obj = (LV2_Atom_Object*)&ev->body;
+		} else if (lv2_atom_forge_is_object_type(&self->forge, ev->body.type)) {
+			const LV2_Atom_Object* obj = (const LV2_Atom_Object*)&ev->body;
 			if (obj->body.otype == uris->patch_Set) {
-				/* Received a set message, send it to the worker. */
-				print(self, self->uris.log_Trace, "Queueing set message\n");
-				self->schedule->schedule_work(self->schedule->handle,
-				                              lv2_atom_total_size(&ev->body),
-				                              &ev->body);
+				// Get the property and value of the set message
+				const LV2_Atom* property = NULL;
+				const LV2_Atom* value    = NULL;
+				lv2_atom_object_get(obj,
+				                    uris->patch_property, &property,
+				                    uris->patch_value,    &value,
+				                    0);
+				if (!property) {
+					lv2_log_error(&self->logger,
+					              "patch:Set message with no property\n");
+					continue;
+				} else if (property->type != uris->atom_URID) {
+					lv2_log_error(&self->logger,
+					              "patch:Set property is not a URID\n");
+					continue;
+				}
+
+				const uint32_t key = ((const LV2_Atom_URID*)property)->body;
+				if (key == uris->eg_sample) {
+					// Sample change, send it to the worker.
+					lv2_log_trace(&self->logger, "Queueing set message\n");
+					self->schedule->schedule_work(self->schedule->handle,
+					                              lv2_atom_total_size(&ev->body),
+					                              &ev->body);
+				} else if (key == uris->param_gain) {
+					// Gain change
+					if (value->type == uris->atom_Float) {
+						self->gain = DB_CO(((LV2_Atom_Float*)value)->body);
+					}
+				}
+			} else if (obj->body.otype == uris->patch_Get) {
+				// Received a get message, emit our state (probably to UI)
+				lv2_log_trace(&self->logger, "Responding to get request\n");
+				lv2_atom_forge_frame_time(&self->forge, self->frame_offset);
+				write_set_file(&self->forge, &self->uris,
+				               self->sample->path,
+				               self->sample->path_len);
 			} else {
-				print(self, self->uris.log_Trace,
-				      "Unknown object type %d\n", obj->body.otype);
+				lv2_log_trace(&self->logger,
+				              "Unknown object type %d\n", obj->body.otype);
 			}
 		} else {
-			print(self, self->uris.log_Trace,
-			      "Unknown event type %d\n", ev->body.type);
+			lv2_log_trace(&self->logger,
+			              "Unknown event type %d\n", ev->body.type);
 		}
 	}
 
-	/* Render the sample (possibly already in progress) */
+	// Render the sample (possibly already in progress)
 	if (self->play) {
 		uint32_t       f  = self->frame;
 		const uint32_t lf = self->sample->info.frames;
@@ -398,7 +416,7 @@ run(LV2_Handle instance,
 		}
 
 		for (; pos < sample_count && f < lf; ++pos, ++f) {
-			output[pos] = self->sample->data[f];
+			output[pos] = self->sample->data[f] * self->gain;
 		}
 
 		self->frame = f;
@@ -408,7 +426,7 @@ run(LV2_Handle instance,
 		}
 	}
 
-	/* Add zeros to end if sample not long enough (or not playing) */
+	// Add zeros to end if sample not long enough (or not playing)
 	for (; pos < sample_count; ++pos) {
 		output[pos] = 0.0f;
 	}
@@ -436,7 +454,7 @@ save(LV2_Handle                instance,
 	char* apath = map_path->abstract_path(map_path->handle, self->sample->path);
 
 	store(handle,
-	      self->uris.eg_file,
+	      self->uris.eg_sample,
 	      apath,
 	      strlen(self->sample->path) + 1,
 	      self->uris.atom_Path,
@@ -462,14 +480,15 @@ restore(LV2_Handle                  instance,
 
 	const void* value = retrieve(
 		handle,
-		self->uris.eg_file,
+		self->uris.eg_sample,
 		&size, &type, &valflags);
 
 	if (value) {
 		const char* path = (const char*)value;
-		print(self, self->uris.log_Trace, "Restoring file %s\n", path);
+		lv2_log_trace(&self->logger, "Restoring file %s\n", path);
 		free_sample(self, self->sample);
 		self->sample = load_sample(self, path);
+		self->sample_changed = true;
 	}
 
 	return LV2_STATE_SUCCESS;
