@@ -21,93 +21,84 @@
 #include "lv2/lv2plug.in/ns/ext/log/log.h"
 #include "lv2/lv2plug.in/ns/ext/midi/midi.h"
 #include "lv2/lv2plug.in/ns/ext/state/state.h"
+#include "lv2/lv2plug.in/ns/ext/parameters/parameters.h"
 
 #define EG_SAMPLER_URI          "http://lv2plug.in/plugins/eg-sampler"
-#define EG_SAMPLER__file        EG_SAMPLER_URI "#file"
+#define EG_SAMPLER__sample      EG_SAMPLER_URI "#sample"
 #define EG_SAMPLER__applySample EG_SAMPLER_URI "#applySample"
 #define EG_SAMPLER__freeSample  EG_SAMPLER_URI "#freeSample"
 
 typedef struct {
-	LV2_URID atom_Blank;
+	LV2_URID atom_Float;
 	LV2_URID atom_Path;
 	LV2_URID atom_Resource;
 	LV2_URID atom_Sequence;
+	LV2_URID atom_URID;
 	LV2_URID atom_eventTransfer;
 	LV2_URID eg_applySample;
-	LV2_URID eg_file;
+	LV2_URID eg_sample;
 	LV2_URID eg_freeSample;
-	LV2_URID log_Error;
-	LV2_URID log_Trace;
 	LV2_URID midi_Event;
+	LV2_URID param_gain;
+	LV2_URID patch_Get;
 	LV2_URID patch_Set;
-	LV2_URID patch_body;
+	LV2_URID patch_property;
+	LV2_URID patch_value;
 } SamplerURIs;
 
 static inline void
 map_sampler_uris(LV2_URID_Map* map, SamplerURIs* uris)
 {
-	uris->atom_Blank         = map->map(map->handle, LV2_ATOM__Blank);
+	uris->atom_Float         = map->map(map->handle, LV2_ATOM__Float);
 	uris->atom_Path          = map->map(map->handle, LV2_ATOM__Path);
 	uris->atom_Resource      = map->map(map->handle, LV2_ATOM__Resource);
 	uris->atom_Sequence      = map->map(map->handle, LV2_ATOM__Sequence);
+	uris->atom_URID          = map->map(map->handle, LV2_ATOM__URID);
 	uris->atom_eventTransfer = map->map(map->handle, LV2_ATOM__eventTransfer);
 	uris->eg_applySample     = map->map(map->handle, EG_SAMPLER__applySample);
-	uris->eg_file            = map->map(map->handle, EG_SAMPLER__file);
 	uris->eg_freeSample      = map->map(map->handle, EG_SAMPLER__freeSample);
-	uris->log_Error          = map->map(map->handle, LV2_LOG__Error);
-	uris->log_Trace          = map->map(map->handle, LV2_LOG__Trace);
+	uris->eg_sample          = map->map(map->handle, EG_SAMPLER__sample);
 	uris->midi_Event         = map->map(map->handle, LV2_MIDI__MidiEvent);
+	uris->param_gain         = map->map(map->handle, LV2_PARAMETERS__gain);
+	uris->patch_Get          = map->map(map->handle, LV2_PATCH__Get);
 	uris->patch_Set          = map->map(map->handle, LV2_PATCH__Set);
-	uris->patch_body         = map->map(map->handle, LV2_PATCH__body);
-}
-
-static inline bool
-is_object_type(const SamplerURIs* uris, LV2_URID type)
-{
-	return type == uris->atom_Resource
-		|| type == uris->atom_Blank;
+	uris->patch_property     = map->map(map->handle, LV2_PATCH__property);
+	uris->patch_value        = map->map(map->handle, LV2_PATCH__value);
 }
 
 /**
  * Write a message like the following to @p forge:
- * [
+ * []
  *     a patch:Set ;
- *     patch:body [
- *         eg-sampler:file </home/me/foo.wav> ;
- *     ] ;
- * ]
+ *     patch:property eg:sample ;
+ *     patch:value </home/me/foo.wav> .
  */
 static inline LV2_Atom*
 write_set_file(LV2_Atom_Forge*    forge,
                const SamplerURIs* uris,
                const char*        filename,
-               const size_t       filename_len)
+               const uint32_t     filename_len)
 {
-	LV2_Atom_Forge_Frame set_frame;
-	LV2_Atom* set = (LV2_Atom*)lv2_atom_forge_blank(
-		forge, &set_frame, 1, uris->patch_Set);
+	LV2_Atom_Forge_Frame frame;
+	LV2_Atom* set = (LV2_Atom*)lv2_atom_forge_object(
+		forge, &frame, 0, uris->patch_Set);
 
-	lv2_atom_forge_property_head(forge, uris->patch_body, 0);
-	LV2_Atom_Forge_Frame body_frame;
-	lv2_atom_forge_blank(forge, &body_frame, 2, 0);
-
-	lv2_atom_forge_property_head(forge, uris->eg_file, 0);
+	lv2_atom_forge_key(forge, uris->patch_property);
+	lv2_atom_forge_urid(forge, uris->eg_sample);
+	lv2_atom_forge_key(forge, uris->patch_value);
 	lv2_atom_forge_path(forge, filename, filename_len);
 
-	lv2_atom_forge_pop(forge, &body_frame);
-	lv2_atom_forge_pop(forge, &set_frame);
+	lv2_atom_forge_pop(forge, &frame);
 
 	return set;
 }
 
 /**
  * Get the file path from a message like:
- * [
+ * []
  *     a patch:Set ;
- *     patch:body [
- *         eg-sampler:file </home/me/foo.wav> ;
- *     ] ;
- * ]
+ *     patch:property eg:sample ;
+ *     patch:value </home/me/foo.wav> .
  */
 static inline const LV2_Atom*
 read_set_file(const SamplerURIs*     uris,
@@ -118,23 +109,28 @@ read_set_file(const SamplerURIs*     uris,
 		return NULL;
 	}
 
-	/* Get body of message. */
-	const LV2_Atom_Object* body = NULL;
-	lv2_atom_object_get(obj, uris->patch_body, &body, 0);
-	if (!body) {
+	/* Get property URI. */
+	const LV2_Atom* property = NULL;
+	lv2_atom_object_get(obj, uris->patch_property, &property, 0);
+	if (!property) {
 		fprintf(stderr, "Malformed set message has no body.\n");
 		return NULL;
-	}
-	if (!is_object_type(uris, body->atom.type)) {
-		fprintf(stderr, "Malformed set message has non-object body.\n");
+	} else if (property->type != uris->atom_URID) {
+		fprintf(stderr, "Malformed set message has non-URID property.\n");
+		return NULL;
+	} else if (((const LV2_Atom_URID*)property)->body != uris->eg_sample) {
+		fprintf(stderr, "Set message for unknown property.\n");
 		return NULL;
 	}
 
-	/* Get file path from body. */
+	/* Get value. */
 	const LV2_Atom* file_path = NULL;
-	lv2_atom_object_get(body, uris->eg_file, &file_path, 0);
+	lv2_atom_object_get(obj, uris->patch_value, &file_path, 0);
 	if (!file_path) {
-		fprintf(stderr, "Ignored set message with no file path.\n");
+		fprintf(stderr, "Malformed set message has no value.\n");
+		return NULL;
+	} else if (file_path->type != uris->atom_Path) {
+		fprintf(stderr, "Set message value is not a Path.\n");
 		return NULL;
 	}
 
